from ftw.builder import Builder
from ftw.builder import create
from ftw.keywordwidget.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import json
import transaction


class TestKeywordWidget(FunctionalTestCase):

    def setUp(self):
        super(TestKeywordWidget, self).setUp()
        self.grant('Manager')
        self.setup_fti()

        # Create a content with some keywords
        self.sample = create(Builder('sample content')
                             .titled(u'Dummy content for keywords')
                             .having(subjects=('F\xc3\xb6\xc3\xb6',
                                               'Bar', 'Baz')))

    @browsing
    def test_widget_render_input(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        self.assertTrue(tags)

        # Possible options
        self.assertListEqual(
            ['Bar', 'Baz', u'F\xf6\xf6'],
            tags.options_labels)

        # New entries field id derives from origin field id.
        new = browser.css('#' + tags.attrib['id'] + '_new')
        self.assertTrue(new)

    @browsing
    def test_select_and_save_from_given_entries(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        form = browser.find_form_by_field('Tags')
        form.fill({'Tags': ('Bar', 'F=C3=B6=C3=B6')}).submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertEquals(('Bar', 'F=C3=B6=C3=B6'), tuple(tags.value))

    @browsing
    def test_add_new_tokens_to_field(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        form = browser.find_form_by_field('Tags')
        new = browser.css('#' + tags.attrib['id'] + '_new').first
        new.text = u'NewItem1\nNew Item 2\nN\xf6i 3'
        form.submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTupleEqual(('New Item 2', 'NewItem1', 'N=C3=B6i 3'),
                              tuple(tags.value))

    @browsing
    def test_add_new_terms_is_only_available_to_certain_roles(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        content.manage_permission('ftw.keywordwidget: Add new term',
                                  roles=[],
                                  acquire=0)
        transaction.commit()

        browser.login().visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertFalse(len(browser.css('#' + tags.attrib['id'] + '_new')),
                         'Add new term field should NOT be there')

        content.manage_permission('ftw.keywordwidget: Add new term',
                                  roles=['Manager'],
                                  acquire=0)
        transaction.commit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTrue(browser.css('#' + tags.attrib['id'] + '_new'),
                        'Add new term field should BE there')

    @browsing
    def test_default_select2_config_on_widget(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')

        select2_config = json.loads(tags.attrib['data-select2config'])
        self.assertListEqual(
            [u'allowClear', u'width', u'noResultFound', u'placeholder'],
            select2_config.keys())

    @browsing
    def test_add_twice_the_same_term_in_fact_adds_only_one(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        form = browser.find_form_by_field('Tags')
        new = browser.css('#' + tags.attrib['id'] + '_new').first
        new.text = u'New\nNew'
        form.submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTupleEqual(('New', ),
                              tuple(tags.value))

    @browsing
    def test_add_new_term_to_a_give_set_of_tags(self, browser):
        content = create(Builder('sample content')
                         .titled(u'A content')
                         .having(subjects=('Bar', 'Baz')))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        form = browser.find_form_by_field('Tags')
        new = browser.css('#' + tags.attrib['id'] + '_new').first
        new.text = u'New'
        form.submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTupleEqual(('Bar', 'Baz', 'New'),
                              tuple(tags.value))

    @browsing
    def test_add_new_value_which_is_already_in_the_source(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        form = browser.find_form_by_field('Tags')
        new = browser.css('#' + tags.attrib['id'] + '_new').first
        new.text = u'Bar'
        form.submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTupleEqual(('Bar', ),
                              tuple(tags.value))

    @browsing
    def test_add_new_term_to_empty_vocabulary(self, browser):
        self.portal.manage_delObjects(ids=[self.sample.getId()])
        transaction.commit()

        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')

        tags = browser.find_field_by_text(u'Tags')
        form = browser.find_form_by_field('Tags')
        new = browser.css('#' + tags.attrib['id'] + '_new').first
        new.text = u'NewItem1\nNew Item 2\nN\xf6i 3'
        form.submit()

        browser.visit(content, view='edit')
        tags = browser.find_field_by_text(u'Tags')
        self.assertTupleEqual(('New Item 2', 'NewItem1', 'N=C3=B6i 3'),
                              tuple(tags.value))



from zope.interface import implementer
from zope.schema import Choice
from zope.schema.interfaces import ConstraintNotSatisfied
from zope.schema.interfaces import IChoice
from zope.schema.interfaces import IFromUnicode
from zope.schema.vocabulary import getVocabularyRegistry
from zope.schema.vocabulary import VocabularyRegistryError


@implementer(IChoice, IFromUnicode)
class ChoicePlus(Choice):
    """A choice field, which allows additional terms.
    Example: Allow new tags for catalog based vocabulary"""

    allow_new = None

    def __init__(self, values=None, vocabulary=None, source=None, **kw):
        super(ChoicePlus, self).__init__(values, vocabulary, source, **kw)
        self.allow_new = True

    def _validate(self, value):
        # Pass all validations during initialization
        if self._init_field:
            return

        super(Choice, self)._validate(value)
        vocabulary = self.vocabulary
        if vocabulary is None:
            vr = getVocabularyRegistry()
            try:
                vocabulary = vr.get(None, self.vocabularyName)
            except VocabularyRegistryError:
                raise ValueError("Can't validate value without vocabulary")

        # The widget can control this attribute too
        if self.allow_new:
            # Allow new values!
            return
        else:
            if value not in vocabulary:
                raise ConstraintNotSatisfied(value)



from ftw.builder import Builder
from ftw.builder import create
from ftw.keywordwidget.tests import FunctionalTestCase
from ftw.testbrowser import browsing
import transaction


class TestKeywordWidget(FunctionalTestCase):

    def setUp(self):
        super(TestKeywordWidget, self).setUp()
        self.grant('Manager')
        additional_behaviors = [
            'ftw.keywordwidget.behavior.IKeywordUseCases',
        ]
        self.setup_fti(additional_behaviors=additional_behaviors)
        transaction.commit()

    @browsing
    def test_choose_from_regular_list_with_choice_widget(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')
        types = browser.find_field_by_text(u'Types')

        self.assertTrue(types)
        # No add new terms field
        self.assertFalse(browser.css('#' + types.attrib['id'] + '_new'))

        # Some Possible options
        self.assertIn('SampleContent',
                      types.options_labels)
        self.assertIn('File',
                      types.options_labels)
        self.assertIn('Link',
                      types.options_labels)

        browser.fill({'Types': ('File', 'SampleContent')}).submit()
        browser.login().visit(content, view='edit')
        types = browser.find_field_by_text(u'Types')
        self.assertEquals(('File', 'SampleContent'), tuple(types.value))

    @browsing
    def test_choose_from_regular_choice_widget(self, browser):
        content = create(Builder('sample content').titled(u'A content'))
        browser.login().visit(content, view='edit')
        types = browser.find_field_by_text(u'Single type')

        self.assertTrue(types)
        # No add new terms field
        self.assertFalse(browser.css('#' + types.attrib['id'] + '_new'))

        # Some Possible options
        self.assertIn('SampleContent',
                      types.options_labels)
        self.assertIn('File',
                      types.options_labels)
        self.assertIn('Link',
                      types.options_labels)

        browser.fill({'Single type': 'SampleContent'}).submit()
        browser.login().visit(content, view='edit')
        types = browser.find_field_by_text(u'Single type')
        self.assertEquals('SampleContent', types.value)



from ftw.builder import registry
from ftw.builder.dexterity import DexterityBuilder
from ftw.keywordwidget.testing import FTW_FUNCTIONAL
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.dexterity.content import Container
from plone.dexterity.fti import DexterityFTI
from plone.supermodel import model
from unittest2 import TestCase
from zope.interface import implements
from zope.interface import Interface
import transaction


class ISampleContentSchema(model.Schema):
    pass


class ISampleContententMarker(Interface):
    pass


class SampleContent(Container):
    implements(ISampleContententMarker)


class FunctionalTestCase(TestCase):
    layer = FTW_FUNCTIONAL

    def setUp(self):
        self.portal = self.layer['portal']
        self.request = self.layer['request']

    def grant(self, *roles):
        setRoles(self.portal, TEST_USER_ID, list(roles))
        transaction.commit()

    def setup_fti(self, additional_behaviors=None):
        types_tool = self.portal.portal_types

        default_behaviors = [
            'plone.app.dexterity.behaviors.metadata.IBasic',
            'plone.app.content.interfaces.INameFromTitle'
        ]

        if additional_behaviors is None:
            default_behaviors += [
                'ftw.keywordwidget.behavior.IKeywordCategorization',
            ]
        else:
            default_behaviors += additional_behaviors

        fti = DexterityFTI('SampleContent')
        fti.schema = 'ftw.keywordwidget.tests.ISampleContentSchema'
        fti.klass = 'ftw.keywordwidget.tests.SampleContent'
        fti.behaviors = tuple(default_behaviors)
        fti.default_view = 'view'
        types_tool._setObject('SampleContent', fti)

        transaction.commit()


# Builders
class SampleContentBuilder(DexterityBuilder):
    portal_type = 'SampleContent'

registry.builder_registry.register(
    'sample content', SampleContentBuilder)



from binascii import b2a_qp
from ftw.keywordwidget import _
from ftw.keywordwidget.field import ChoicePlus
from plone import api
from Products.CMFPlone.utils import safe_unicode
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form.browser.select import SelectWidget
from z3c.form.interfaces import DISPLAY_MODE
from z3c.form.interfaces import HIDDEN_MODE
from z3c.form.interfaces import IFieldWidget
from z3c.form.interfaces import INPUT_MODE
from z3c.form.interfaces import NOVALUE
from z3c.form.widget import FieldWidget
from zope import schema
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary
import json


class KeywordWidget(SelectWidget):

    klass = u'keyword-widget'

    noValueToken = u''

    noValueMessage = _('no value')
    promptMessage = _('select some values ...')
    promptNoresultFound = _('No result found')
    multiple = 'multiple'
    size = 10

    # KeywordWidget specific
    js_config = None
    choice_field = None

    display_template = ViewPageTemplateFile('templates/keyword_display.pt')
    input_template = ViewPageTemplateFile('templates/keyword_input.pt')
    hidden_template = ViewPageTemplateFile('templates/keyword_hidden.pt')
    js_template = ViewPageTemplateFile('templates/keyword.js.pt')

    def __init__(self, request, js_config=None):
        self.request = request
        self.js_config = js_config

    def render(self):
        if self.mode == INPUT_MODE:
            return self.input_template(self)
        elif self.mode == DISPLAY_MODE:
            return self.display_template(self)
        elif self.mode == HIDDEN_MODE:
            return self.hidden_template(self)
        raise NotImplementedError(
            'Mode: "{0}" not supported'.format(self.mode))

    def get_new_values_from_request(self):
        """Get the values from the request and splits addition values by
        newlines,
        """
        new_values = set()
        new = self.request.get('{0}_new'.format(self.name), '').split('\n')
        for value in new:
            cleanedup_value = value.strip().strip('\r').strip('\n')
            if cleanedup_value:
                new_values.add(cleanedup_value)
        return list(new_values)

    def update(self):
        super(KeywordWidget, self).update()

        self.update_multivalued_property()
        self.get_choice_field()
        self.update_js_config()

        if isinstance(self.choice_field, ChoicePlus):
            has_permission = api.user.has_permission(
                'ftw.keywordwidget: Add new term',
                obj=self.context)
            self.field.value_type.allow_new = has_permission

    def update_js_config(self):
        # Sane default config
        default_config = {
            'placeholder': translate(self.promptMessage, context=self.request),
            'noResultFound': translate(self.promptNoresultFound,
                                       context=self.request),
            'width': '300px',
            'allowClear': not self.field.required and not self.multiple,
            # 'tags': self.show_add_term_field()
        }

        if self.js_config:
            default_config.update(self.js_config)

        self.js_config = json.dumps(default_config)

    def update_multivalued_property(self):
        if not isinstance(self.field, schema.List):
            self.multiple = None
            self.size = 1
            self.promptMessage = _('select a value ...')

    def get_choice_field(self):
        is_list = isinstance(self.field, schema.List)
        is_choice = isinstance(self.field, schema.Choice)

        if is_list and isinstance(self.field.value_type, schema.Choice):
            self.choice_field = self.field.value_type
        elif is_choice:
            self.choice_field = self.field
        else:
            raise ValueError('The field or the value type of the field '
                             'needs to be a Choice field.')

    def show_add_term_field(self):
        if isinstance(self.choice_field, ChoicePlus):
            return self.field.value_type.allow_new
        else:
            return False

    def extract(self, default=NOVALUE):
        """See z3c.form.interfaces.IWidget.
        """
        values = super(KeywordWidget, self).extract(default=default)

        if not self.show_add_term_field():
            return values

        else:
            # Adding new keywords, which are not in the vocab/source
            if (self.name + '_new' not in self.request and
                    self.name + '-empty-marker' in self.request):
                return []

            new_values = self.get_new_values_from_request()

            if values is default:
                values = []
            else:
                values = list(values)

            for new_value in new_values:
                # The new values needs to fit the token value in the
                # vocabulary
                if isinstance(new_value, unicode):
                    new_value = new_value.encode('utf-8')
                    new_value = b2a_qp(new_value)
                    values.append(new_value)

            return values and values or default

    def updateTerms(self):
        super(KeywordWidget, self).updateTerms()
        simple_vocbaulary = self.terms.terms
        terms = self.terms.terms._terms

        for new_value in self.get_new_values_from_request():
            if new_value not in simple_vocbaulary.by_value:

                # Vocabulary term tokens *must* be 7 bit values, titles *must*
                # be unicode.
                # Value needs to be a utf-8 str, hell I don't know why.
                # IMHO this should depend on the source. We're gonna have
                # trouble with this in the future.
                new_token = new_value
                if isinstance(new_value, unicode):
                    new_value = new_token = new_value.encode('utf-8')
                new_token = b2a_qp(new_token)
                terms.append(
                    SimpleTerm(new_value, new_token, safe_unicode(new_value)))

        self.terms.terms = SimpleVocabulary(terms)
        return self.terms

    def keyword_js(self):
        return self.js_template(widgetid=self.id)


@implementer(IFieldWidget)
def KeywordFieldWidget(field, request):
    return FieldWidget(field, KeywordWidget(request))
